@@ -42,11 +42,18 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 #include <jpeglib.h>
 #include "tran-supp.h"      /* Support routines for jpegtran */
 #include <limits.h>
 #include <errno.h>
+#include <glib.h>
+#include <glib/gstdio.h>
+
+int jpegtran (char *input_filename, char *output_filename, JXFORM_CODE  transformation);
+
+int rotate_and_save_jpeg_lossless(char *  filename,int angle);
 
 int
 jpegtran (char        *input_filename,
